@@ -25,23 +25,7 @@ with zeroconf.
 This also means we don't need to add any extra dependencies.
 """
 
-import asyncio
-from functools import partial
-
-from zeroconf import Zeroconf
-
-from aiohomekit.zeroconf import discover_homekit_devices
-
-
-async def async_discover_homekit_devices(
-    max_seconds=10, zeroconf_instance: "Zeroconf" = None
-):
-    loop = asyncio.get_event_loop()
-    return await loop.run_in_executor(
-        None,
-        partial(
-            discover_homekit_devices,
-            max_seconds=max_seconds,
-            zeroconf_instance=zeroconf_instance,
-        ),
-    )
+from aiohomekit.zeroconf import (  # noqa: F401
+    async_discover_homekit_devices,
+    discover_homekit_devices,
+)
